@@ -1,24 +1,43 @@
 """Create a tmux workspace from a workspace :py:obj:`dict`."""
 
+from __future__ import annotations
+
 import logging
+import os
+import shutil
 import time
 import typing as t
 
 from libtmux._internal.query_list import ObjectDoesNotExist
-from libtmux.common import has_gte_version, has_lt_version
 from libtmux.pane import Pane
 from libtmux.server import Server
 from libtmux.session import Session
 from libtmux.window import Window
 
-from .. import exc
-from ..util import get_current_pane, run_before_script
+from tmuxp import exc
+from tmuxp.util import get_current_pane, run_before_script
+
+if t.TYPE_CHECKING:
+    from collections.abc import Iterator
 
 logger = logging.getLogger(__name__)
 
-DEFAULT_WIDTH = "800"
-DEFAULT_HEIGHT = "600"
-DEFAULT_SIZE = f"{DEFAULT_WIDTH}x{DEFAULT_HEIGHT}"
+COLUMNS_FALLBACK = 80
+
+
+def get_default_columns() -> int:
+    """Return default session column size use when building new tmux sessions."""
+    return int(
+        os.getenv("TMUXP_DEFAULT_COLUMNS", os.getenv("COLUMNS", COLUMNS_FALLBACK)),
+    )
+
+
+ROWS_FALLBACK = int(os.getenv("TMUXP_DEFAULT_ROWS", os.getenv("ROWS", 24)))
+
+
+def get_default_rows() -> int:
+    """Return default session row size use when building new tmux sessions."""
+    return int(os.getenv("TMUXP_DEFAULT_ROWS", os.getenv("ROWS", ROWS_FALLBACK)))
 
 
 class WorkspaceBuilder:
@@ -128,15 +147,15 @@ class WorkspaceBuilder:
     a session inside tmux (when `$TMUX` is in the env variables).
     """
 
-    server: "Server"
-    _session: t.Optional["Session"]
+    server: Server
+    _session: Session | None
     session_name: str
 
     def __init__(
         self,
-        session_config: t.Dict[str, t.Any],
+        session_config: dict[str, t.Any],
         server: Server,
-        plugins: t.Optional[t.List[t.Any]] = None,
+        plugins: list[t.Any] | None = None,
     ) -> None:
         """Initialize workspace loading.
 
@@ -159,7 +178,7 @@ class WorkspaceBuilder:
         if plugins is None:
             plugins = []
         if not session_config:
-            raise exc.EmptyWorkspaceException()
+            raise exc.EmptyWorkspaceException
 
         # validation.validate_schema(session_config)
 
@@ -185,7 +204,7 @@ class WorkspaceBuilder:
     def session(self) -> Session:
         """Return tmux session using in workspace builder session."""
         if self._session is None:
-            raise exc.SessionMissingWorkspaceException()
+            raise exc.SessionMissingWorkspaceException
         return self._session
 
     def session_exists(self, session_name: str) -> bool:
@@ -203,7 +222,7 @@ class WorkspaceBuilder:
             return False
         return True
 
-    def build(self, session: t.Optional[Session] = None, append: bool = False) -> None:
+    def build(self, session: Session | None = None, append: bool = False) -> None:
         """Build tmux workspace in session.
 
         Optionally accepts ``session`` to build with only session object.
@@ -220,23 +239,30 @@ class WorkspaceBuilder:
         """
         if not session:
             if not self.server:
-                raise exc.TmuxpException(
+                msg = (
                     "WorkspaceBuilder.build requires server to be passed "
-                    + "on initialization, or pass in session object to here.",
+                    "on initialization, or pass in session object to here."
                 )
-            else:
-                new_session_kwargs = {}
-                if "start_directory" in self.session_config:
-                    new_session_kwargs["start_directory"] = self.session_config[
-                        "start_directory"
-                    ]
-                if has_gte_version("2.6"):
-                    new_session_kwargs["x"] = 800
-                    new_session_kwargs["y"] = 600
-                session = self.server.new_session(
-                    session_name=self.session_config["session_name"],
-                    **new_session_kwargs,
+                raise exc.TmuxpException(
+                    msg,
                 )
+            new_session_kwargs = {}
+            if "start_directory" in self.session_config:
+                new_session_kwargs["start_directory"] = self.session_config[
+                    "start_directory"
+                ]
+
+            if os.getenv("TMUXP_DETECT_TERMINAL_SIZE", "1") == "1":
+                terminal_size = shutil.get_terminal_size(
+                    fallback=(get_default_columns(), get_default_rows()),
+                )
+                new_session_kwargs["x"] = terminal_size.columns
+                new_session_kwargs["y"] = terminal_size.lines
+
+            session = self.server.new_session(
+                session_name=self.session_config["session_name"],
+                **new_session_kwargs,
+            )
             assert session is not None
 
             assert self.session_config["session_name"] == session.name
@@ -249,7 +275,7 @@ class WorkspaceBuilder:
 
         assert session.server is not None
 
-        self.server: "Server" = session.server
+        self.server: Server = session.server
         assert self.server.sessions is not None
         assert self.server.has_session(session.name)
         assert session.id
@@ -280,7 +306,7 @@ class WorkspaceBuilder:
 
         if "global_options" in self.session_config:
             for option, value in self.session_config["global_options"].items():
-                self.session.set_option(option, value, _global=True)
+                self.session.set_option(option, value, global_=True)
 
         if "environment" in self.session_config:
             for option, value in self.session_config["environment"].items():
@@ -321,7 +347,7 @@ class WorkspaceBuilder:
         self,
         session: Session,
         append: bool = False,
-    ) -> t.Iterator[t.Any]:
+    ) -> Iterator[t.Any]:
         """Return :class:`libtmux.Window` iterating through session config dict.
 
         Generator yielding :class:`libtmux.Window` by iterating through
@@ -376,22 +402,6 @@ class WorkspaceBuilder:
                 pass
 
             environment = panes[0].get("environment", window_config.get("environment"))
-            if environment and has_lt_version("3.0"):
-                # Falling back to use the environment of the first pane for the window
-                # creation is nice but yields misleading error messages.
-                pane_env = panes[0].get("environment")
-                win_env = window_config.get("environment")
-                if pane_env and win_env:
-                    target = "panes and windows"
-                elif pane_env:
-                    target = "panes"
-                else:
-                    target = "windows"
-                logger.warning(
-                    f"Cannot set environment for new {target}. "
-                    "You need tmux 3.0 or newer for this.",
-                )
-                environment = None
 
             window = session.new_window(
                 window_name=window_name,
@@ -411,7 +421,7 @@ class WorkspaceBuilder:
                 dict,
             ):
                 for key, val in window_config["options"].items():
-                    window.set_window_option(key, val)
+                    window.set_option(key, val)
 
             if window_config.get("focus"):
                 window.select()
@@ -421,8 +431,8 @@ class WorkspaceBuilder:
     def iter_create_panes(
         self,
         window: Window,
-        window_config: t.Dict[str, t.Any],
-    ) -> t.Iterator[t.Any]:
+        window_config: dict[str, t.Any],
+    ) -> Iterator[t.Any]:
         """Return :class:`libtmux.Pane` iterating through window config dict.
 
         Run ``shell_command`` with ``$ tmux send-keys``.
@@ -442,9 +452,8 @@ class WorkspaceBuilder:
         """
         assert isinstance(window, Window)
 
-        pane_base_index_str = window.show_window_option("pane-base-index", g=True)
-        assert pane_base_index_str is not None
-        pane_base_index = int(pane_base_index_str)
+        pane_base_index = window.show_option("pane-base-index", global_=True)
+        assert pane_base_index is not None
 
         pane = None
 
@@ -457,41 +466,29 @@ class WorkspaceBuilder:
             else:
 
                 def get_pane_start_directory(
-                    pane_config: t.Dict[str, str],
-                    window_config: t.Dict[str, str],
-                ) -> t.Optional[str]:
+                    pane_config: dict[str, str],
+                    window_config: dict[str, str],
+                ) -> str | None:
                     if "start_directory" in pane_config:
                         return pane_config["start_directory"]
-                    elif "start_directory" in window_config:
+                    if "start_directory" in window_config:
                         return window_config["start_directory"]
-                    else:
-                        return None
+                    return None
 
                 def get_pane_shell(
-                    pane_config: t.Dict[str, str],
-                    window_config: t.Dict[str, str],
-                ) -> t.Optional[str]:
+                    pane_config: dict[str, str],
+                    window_config: dict[str, str],
+                ) -> str | None:
                     if "shell" in pane_config:
                         return pane_config["shell"]
-                    elif "window_shell" in window_config:
+                    if "window_shell" in window_config:
                         return window_config["window_shell"]
-                    else:
-                        return None
+                    return None
 
                 environment = pane_config.get(
                     "environment",
                     window_config.get("environment"),
                 )
-                if environment and has_lt_version("3.0"):
-                    # Just issue a warning when the environment comes from the pane
-                    # configuration as a warning for the window was already issued when
-                    # the window was created.
-                    if pane_config.get("environment"):
-                        logger.warning(
-                            "Cannot set environment for new panes. "
-                            "You need tmux 3.0 or newer for this.",
-                        )
-                    environment = None
 
                 assert pane is not None
 
@@ -545,7 +542,7 @@ class WorkspaceBuilder:
     def config_after_window(
         self,
         window: Window,
-        window_config: t.Dict[str, t.Any],
+        window_config: dict[str, t.Any],
     ) -> None:
         """Actions to apply to window after window and pane finished.
 
@@ -565,7 +562,7 @@ class WorkspaceBuilder:
             dict,
         ):
             for key, val in window_config["options_after"].items():
-                window.set_window_option(key, val)
+                window.set_option(key, val)
 
     def find_current_attached_session(self) -> Session:
         """Return current attached session."""
@@ -574,7 +571,7 @@ class WorkspaceBuilder:
         current_active_pane = get_current_pane(self.server)
 
         if current_active_pane is None:
-            raise exc.ActiveSessionMissingWorkspaceException()
+            raise exc.ActiveSessionMissingWorkspaceException
 
         return next(
             (
